//-------------------------------------------------------------------------------------------
#ifndef __PHOENIX_TS_WATERDEMO_WATERLBCOMPUTATION_H
#define __PHOENIX_TS_WATERDEMO_WATERLBCOMPUTATION_H
//-------------------------------------------------------------------------------------------
#ifdef __cplusplus
extern "C" {
#endif
//-------------------------------------------------------------------------------------------

typedef struct
{

	int nx;
	int ny;
	float *heightMap;
	float *uVelocityMap;
	float *vVelocityMap;
	// force is an interleaved memory array [m_ny][m_nx][9]
	// 128-byte aligned memory access for each block
	float *force9DMap;
	float *forceInc9DMap;
	float *fEQ9DMap;
	float *forceX;
	float *forceY;
	unsigned int textureId;
	unsigned int pboId;
	struct cudaGraphicsResource *glResource;
	unsigned char *imageData;
} WaterLBComputeData;

//-------------------------------------------------------------------------------------------

bool waterLBResolveHUV(WaterLBComputeData *pData);

bool waterLBComputeFEQ(WaterLBComputeData *pData);

bool waterLBCollision(WaterLBComputeData *pData, bool resolveFlag);

bool waterLBCollisionProcess(WaterLBComputeData *pData, bool resolveFlag);

bool waterLBStream(WaterLBComputeData *pData);

void waterLBSwap(WaterLBComputeData *pData);

bool waterLBIncrementForce(WaterLBComputeData *pData);

bool waterLBRaindropCircleForce(WaterLBComputeData *pData, float posX, float posY, float rForce, float rRadius);

WaterLBComputeData *waterLBAllocateAndInit(int nx, int ny);

void waterLBFree(WaterLBComputeData *pData);

void waterLBGetHeightMap(WaterLBComputeData *pData, float *h);

void waterLBPopulateModel(WaterLBComputeData *pData, float **fEQ, float **f, float **fInc, float *h, float *u, float *v, float *forceX, float *forceY);

void waterLBBreakpoint(WaterLBComputeData *pData);

bool waterLBCollisionProcess2(WaterLBComputeData *pData, bool resolveFlag);

bool waterLBCollisionProcess3(WaterLBComputeData *pData, bool resolveFlag);

bool waterLBRenderHeightTexture(WaterLBComputeData *pData);

bool waterLBSetupTextureOutput(WaterLBComputeData *pData,unsigned int texId);

bool waterLBSetupPBOOutput(WaterLBComputeData *pData, unsigned int pboId);

//-------------------------------------------------------------------------------------------
#ifdef __cplusplus
}
#endif
//-------------------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------------------
